/***********************************************************************
 * Author: 	Kevin Chan
 *
 * Description:
 * 		The code present provides an example of the serial interface
 * 		that is used. These are light wrappers in the sense that error
 *      codes are returned directly back to you. You can control the
 *      serial port to if you want as the reference number will be
 *      returned to you.
 *
 *      The only safe condition would possibly be to configure the port;
 *
 * NOTE: The API will multiplex between the old and new Serial Managers
 * if the new Serial Manager is present, that will be the prefered 
 * Manager.
 *
 * It is also dependent on AppStdIO for debugging. Because pre m500
 * devices only have one UART, we cannot live debug the system. So
 * we create a console equivalent (i.e. AppStdIO).
 ***********************************************************************/
#include "serial_ir.h"
#include <SerialMgrOld.h>
#include <SerialMgr.h>

#include "SystemMgr.h"

#ifdef __DEBUG_IR_MSGS
	#include "AppStdIO.h"
	
	#define DEBUG_PRINT_LINE(error) \
		if (error) printf("DEBUG: %s. %d -- Err: %d\r", __FILE__, __LINE__, error);
#else
	#define DEBUG_PRINT_LINE(error)
	#ifndef __APPSTDIO_H__
		#define debug_msg(x)
		#define debug_msg_1(x, y)
	#endif
#endif

static UInt16 irPortId = 0;

static Boolean serIsOpen = false;
static Boolean serMgrNew = false;

/***********************************************************************
 *
 * FUNCTION:     DebugDisplaySerErrMsg
 *
 * DESCRIPTION:  Converts a Serial Error Message and displays the textual
 * equivalent.
 *
 * PARAMETERS:   --> serErr  // The Serial Error Number
 *
 * RETURNED:     Err         // Returns the serErr it received.
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
Err DebugDisplaySerErrMsg(Err serErr)
{
	if (serErr == 0)
		return serErr;
		
	switch (serErr)
	{
		case serErrBadParam:
			debug_msg("Serial Err: serErrBadParam");
			break;
			
		case serErrBadPort:
			debug_msg("Serial Err: serErrBadPort");
			break;
			
		case serErrNoMem:
			debug_msg("Serial Err: serErrNoMem");
			break;
			
		case serErrBadConnID:
			debug_msg("Serial Err: serErrBadConnID");
			break; 
			
		case serErrTimeOut:
			debug_msg("Serial Err: serErrTimeOut");
			break;
			
		case serErrLineErr:
			debug_msg("Serial Err: serErrLineErr");
			break;
			
		case serErrAlreadyOpen:
			debug_msg("Serial Err: serErrAlreadyOpen");
			break;
			
		case serErrStillOpen:
			debug_msg("Serial Err: serErrStillOpen");
			break;
			
		case serErrNotOpen:
			debug_msg("Serial Err: serErrNotOpen");
			break;
			
		case serErrNotSupported:
			debug_msg("Serial Err: serErrNotSupported");
			break;
	
		default:
			debug_msg_1("Unknown SerErr Code: %d", serErr);
	}
	
	return serErr;
}

/***********************************************************************
 *
 * FUNCTION:     DebugDisplayLineErrMsg
 *
 * DESCRIPTION:  Given a line err number, it will convert it to the
 * text equivalent and output a message.
 *
 * NOTE: This function does not clear the line errors. You must
 * clear them before continuing with a call to get data from the UART.
 *
 * PARAMETERS:   UInt16    // the Line Err number
 *
 * RETURNED:     UInt16    // The lineErr number.
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
UInt16 DebugDisplayLineErrMsg(UInt16 lineErrs)
{	
	if (lineErrs & serLineErrorParity)
		debug_msg("Line Error: parity error");
	
	if (lineErrs & serLineErrorHWOverrun)
		debug_msg("Line Error: HW overrun");
		
	if (lineErrs & serLineErrorFraming)
		debug_msg("Line Error: framing error");
		
	if (lineErrs & serLineErrorBreak)
		debug_msg("Line Error: break signal asserted");
		
	if (lineErrs & serLineErrorHShake)
		debug_msg("Line Error: line hand-shake error");
		
	if (lineErrs & serLineErrorSWOverrun)
		debug_msg("Line Error: SW overrun");
		
	return lineErrs;
}


/***********************************************************************
 *
 * FUNCTION:     IrOpenPort
 *
 * DESCRIPTION:  It will attempt to bind itself to a serial line through
 * the UART. More specifically, it will bind to the IR Port. It will
 * also determine whether the New Serial Manager is present and if it
 * is then it will use that library instead.
 *
 * PARAMETERS:   --> baud    // The baud rate you want on the port
 *
 * RETURNED:    errNone				 // No error
 *				serErrAlreadyOpen	 // Port is already opened and port
 *									 // port sharing is not allowed.
 *				serErrBadParam		 // baud rate was invalid.
 *				memErrNotEnoughSpace // Insufficient memory
 *				
 *
 ***********************************************************************/
Err IrOpenPort(UInt32 baud)
{
	Err err = 0;
	UInt32 value = 0;
	
	if (serIsOpen)
		return err;
	
	// See if the new serial manager code is installed.
	err = FtrGet(sysFileCSerialMgr, sysFtrNewSerialPresent, &value);
	
	if (err == 0 && value != 0)
	{
		
		UInt32 size = sizeof(UInt32);
		UInt32 settings = srmSettingsFlagStopBits1 | srmSettingsFlagBitsPerChar8;
		
		// the New Serial Manager (v1) is installed. so we will take
		// advantage of this.
		err = SrmOpen(serPortIrPort, baud, &irPortId);
		DEBUG_PRINT_LINE(err);
		
		debug_msg("Found new print manager");
					
		if (err)
		{
			goto Exit;
		}
				
	   	err = SrmControl(irPortId, srmCtlIrDAEnable, 0L, 0L);
	   	DEBUG_PRINT_LINE(err);
	   	
	   	serMgrNew = true;
	   	
	   	goto Exit;
	}
	
	debug_msg("Old Serial Manager starting");
	// For The old serial Manager, the port number
	// is always equal to Zero;
	if ((err = SysLibFind("Serial Library", &irPortId)) != 0)
	{
		DEBUG_PRINT_LINE(err);
		return err;
	}
		
	err = SerOpen(irPortId, 0, baud);
	
	if (err != 0)
	{
		if (err == serErrAlreadyOpen)
		{
			// We are allowed to share the serial port. That is why this error
			// message is returned. When it is returned the internal serial port
			// open counter is incremented But in because we do not want to
			// share the port (we want exclusive access.) So we must explicitly
			// call close to decrement the counter. we will then return;
			err = SerClose(irPortId);
			DEBUG_PRINT_LINE(err);
		}
		
		goto Exit;
	}
	

	err = SerControl(irPortId, serCtlIrDAEnable, NULL, 0);
	DEBUG_PRINT_LINE(err);

	if (err == 0)
	{		
		//XXX HACK XXX
		// Must be present on Version 3.0 or under
		// Contrary to the KB, we are receiving an extra byte
		// on startup for V3.0
		SerSendFlush(irPortId);
		SerReceiveFlush(irPortId, 0);
	}

Exit:
	debug_msg("Serial Manager started");
	if (!err)
		serIsOpen = true;
		
	return DebugDisplaySerErrMsg(err);	
}


/***********************************************************************
 *
 * FUNCTION:     IrGetSerialLibRefNum
 *
 * DESCRIPTION:  Returns the IR Port ID. The Port ID is serial manager
 * specific. If the new serial manager is present, then the Port ID is
 * should be used as the "portID" parameter for the New Serial Manager
 * APIs.
 *
 * Else, the Port ID should be used as the refNum parameter for the Old
 * Serial Manager APIs (only if the New Serial Manager is not present.)

 * This is useful if you want/need to change the IR port configurations
 *
 * RETURNED:     UInt16  // The library reference number.
 ***********************************************************************/
UInt16 GetIrPortID()
{
	return irPortId;
}

/***********************************************************************
 *
 * FUNCTION:     IsUsingNewSerialManager
 *
 * DESCRIPTION:  Helps determine which serial manager is being used.
 *
 * PARAMETERS:   True (new serial manager v1 is being used)
 *               False (old serial manger is being used.)
 *
 * RETURNED:     UInt16  // The library reference number.
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
Boolean IsUsingNewSerialManager()
{
	return serMgrNew;
}

/***********************************************************************
 *
 * FUNCTION:     IrSendData
 *
 * DESCRIPTION:  A wrapper that wraps the SerSend functionality.
 * Specifically it will send the data you specify and then wait for all
 * of the data to be sent before returnning.
 *
 * NOTE: delivery is not guaranteed. In another words if the receiver
 * experiences a line error (oh well). If you want reliable delivery
 * use the other serial port functionality provided by the PALM or right
 * your own stack on top of these APIs.
 *
 * NOTE 2: On older palms, there is a feedback problem. Thus a call is
 * made to flush the internal receive buffer.
 *
 * PARAMETERS:   -->buffP    // ptr to the buffer containing the data.
 *               -->count    // size of buffP in bytes.
 *               <--bytesTxd // The number of bytes that were sent.
 *
 * RETURNED:     Err       // Any errors that may have been generated by
 *                         // the Ser Functions.
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
Err IrSendData(void* buffP, UInt32 count, UInt32* bytesTxd )
{
	Err err = 0;
	*bytesTxd = 0;
	
	if (!serIsOpen)
	{
		err = serErrNotOpen;
		DEBUG_PRINT_LINE(err);
		goto Exit;
	}
	
	if (serMgrNew)
	{
		///////////////////////////////////////////////////////////
		// The following is the **NEW** Serial Manager Interface
		///////////////////////////////////////////////////////////

		// You may need to disable the serial port for some Palm
		// models. But not for m500s as srmCtlRxDisable is an
		// unrecognized op code.
		// err = SrmControl(irPortId, srmCtlRxDisable, 0L, 0L);
		// DEBUG_PRINT_LINE(err);
		
		if (err) goto Exit;
		
		*bytesTxd = SrmSend(irPortId, buffP, count, &err);
		DEBUG_PRINT_LINE(err);
		
		if (err) goto Exit;

		// Wait until all data has been sent.
		err = SrmSendWait(irPortId);
		DEBUG_PRINT_LINE(err);
		
		goto Exit;
	}	
	else
	///////////////////////////////////////////////////////////
	// The following is the **Old** Serial Manager Interface
	///////////////////////////////////////////////////////////
	{
		*bytesTxd = SerSend(irPortId, buffP, count, &err);
		
		// Now wait until the send has completed
		err = SerSendWait(irPortId, 0);
		
		// Make a call to ReceiveFlush to flush the local receive 
		// queue. This removes the feedback phenomenon that you see
		// when trying to send using the older SerialMgr. (The newer
		// SerialMgr uses a different mechanism.)
		SerReceiveFlush(irPortId, 0);
	}	
	
Exit:

	if (serMgrNew)
		SrmControl(irPortId, srmCtlRxEnable, 0L, 0L);
	
	return DebugDisplaySerErrMsg(err);
}

/***********************************************************************
 *
 * FUNCTION:     IrReceiveFlush
 *
 * DESCRIPTION:  A wrapper around the SerReceiveFlush function
 *
 * PARAMETERS:   -->timeout  // After the timeout amount, SerReceiveFlush
 *                           // will execute.
 *
 * RETURNED:     None
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
void IrReceiveFlush(UInt32 timeout)
{
	if (!serIsOpen)
		return;
		
	if (serMgrNew)
		SrmReceiveFlush(irPortId, timeout);
	else
		SerReceiveFlush(irPortId, timeout);		
}

/***********************************************************************
 *
 * FUNCTION:     IrReceiveData
 *
 * DESCRIPTION:  This will attempt to read whatever is in the receive
 * buffer of the UART. It wraps the SerReceive Functionality.
 * Specifically, it will block until count bytes or a timeout occurred.
 *
 * if a timeout occurs first, we will return whatever bytes is sitting
 * in the buffer.
 *
 * PARAMETERS:   <->buffP     // the buffer to contain the received data.
 *               -->count     // Size of buffP in bytes
 *               <--bytesRcvd // The # of bytes that was received.
 *               -->timeout   // Timeout amount in system ticks. Actual
 *               <--lineErrCode // The Line Error code if a line error occurred.
 *
 * RETURNED:     errNone		// No Error
 *				 serErrNotOpen 	// You forgot to call IrOpenPort
 *				 serErrLineErr	// An RS-232 Line error occurred. See
 *								// lineErrCode parameter for the code.
 *				 serErrNotSupported // You changed the port to a background port
 *
 *				 serErrBadParam	// The bytes parameter exceeds the size of
 *								// Receive buffer queue.
 *
 * NOTE: for the old serial manager, input buffer is only 512 bytes. so
 * if you want more than 512 bytes, you are out of luck. If you try to 
 * pass more, the code will scale this back down to 512.
 *
 * Same thing goes for the new serial manager unless you change the
 * input buffer which you can do only in the new serial manager. You must
 * make this call yourself using SrmSetReceiveBuffer.
 ***********************************************************************/
Err IrReceiveData(void* buffP, UInt32 count, UInt32* bytesRcvd, UInt32 timeout, UInt16 *lineErrCode)
{
	Err err = 0;
	UInt32 numBytes = MAX_RCV_QUEUE_SIZE;

	*bytesRcvd = 0;
	*lineErrCode = 0;
	
	if (!serIsOpen)
	{
		err = serErrNotOpen;
		DEBUG_PRINT_LINE(err);
		goto Exit;
	}

	if (serMgrNew)
	{	
		///////////////////////////////////////////////////////////
		// The following is the **New** Serial Manager Interface
		///////////////////////////////////////////////////////////
		
		// Make sure that the receiver is on.
		SrmControl(irPortId, srmCtlRxEnable, 0L, 0L);
		
		err = SrmReceiveWait(irPortId, count, timeout);
		
		if (err != serErrTimeOut && err != 0)
		{
			DEBUG_PRINT_LINE(err);
			goto Exit;
		}
		
		err = SrmReceiveCheck(irPortId, & numBytes);
		
		// Overflow check
		if (numBytes > count)
			numBytes = count;
			
		(*bytesRcvd) = SrmReceive(irPortId, buffP, numBytes, timeout, &err);

	}
	
	else
	///////////////////////////////////////////////////////////
	// The following is the **Old** Serial Manager Interface
	///////////////////////////////////////////////////////////	
	{
		if (numBytes < count)
			numBytes = count;
					
		// Block and wait until we receive the requested number of bytes
		// (namely until we receive count bytes.)
		err = SerReceiveWait(irPortId, numBytes, timeout);
		
		
		// Even if we timeout, we may received something, so we will
		// go ahead and send this out.
		if (err != serErrTimeOut && err != 0)
		{
			DEBUG_PRINT_LINE(err);
			goto Exit;
		}
		
		
		// There is a case that between the time of the ReceiveWait and
		// the time of ReceiveCheck, we may have received more data than
		// we can actually return.
		err = SerReceiveCheck(irPortId, &numBytes);
	
		if (numBytes > count)
			numBytes = count;
		
		(*bytesRcvd) = SerReceive(irPortId, buffP, numBytes, timeout, &err);
	}
	
	if (err == serErrTimeOut)
		return 0;
		
	DEBUG_PRINT_LINE(err);
	
Exit:		

	// We need to check the line err and handle it accordingly.
	// If we do not clear it, the serErrLineErr will continuously
	// be returned thus giving us false errors.
	if (err == serErrLineErr && lineErrCode)
	{
		if (serMgrNew)
		{
			UInt32 statusField;
			SrmGetStatus(irPortId, &statusField, lineErrCode);
			SrmClearErr(irPortId);
		}
		else
		{
			Boolean ctsOn, dsrOn;
	
			*lineErrCode = SerGetStatus(irPortId, &ctsOn, &dsrOn);
			SerClearErr(irPortId);
		}
		
		DebugDisplayLineErrMsg(*lineErrCode);
	}
	
	return DebugDisplaySerErrMsg(err);
}

/***********************************************************************
 *
 * FUNCTION:     IrReceiveCheck
 *
 * DESCRIPTION:  Light wrapper around the (Srm/Ser)ReceiveCheck function
 *
 * PARAMETERS:   <--bytes   // The number of bytes in the receive buffer.
 *
 * RETURNED:     errNone		 // No Error
 *				 serErrNotOpen   // You forgot to call IrOpenPort
 *				 serErrLineError // A Line Error Occurred, check the
 *								 // lineErrCode paramater
 ***********************************************************************/
Err IrReceiveCheck(UInt32* bytes, UInt16* lineErrCode)
{
	Err err = 0;
	
	if (!serIsOpen)
	{
		err = serErrNotOpen;
		DEBUG_PRINT_LINE(err);
	}
	else
	{
		if (serMgrNew)
			err = SrmReceiveCheck(irPortId, bytes);
		else
			err = SerReceiveCheck(irPortId, bytes);
			
		DEBUG_PRINT_LINE(err);
	}
	
	// We need to check the line err and handle it accordingly.
	// If we do not clear it, the serErrLineErr will continuously
	// be returned thus giving us false errors.
	if (err == serErrLineErr && lineErrCode)
	{
		if (serMgrNew)
		{
			UInt32 statusField;
			SrmGetStatus(irPortId, &statusField, lineErrCode);
			SrmClearErr(irPortId);
		}
		else
		{
			Boolean ctsOn, dsrOn;
	
			*lineErrCode = SerGetStatus(irPortId, &ctsOn, &dsrOn);
			SerClearErr(irPortId);
		}
		
		DebugDisplayLineErrMsg(*lineErrCode);
	}
		
	return DebugDisplaySerErrMsg(err);
}

/***********************************************************************
 *
 * FUNCTION:     IrClosePort
 *
 * DESCRIPTION:  Releases all of the data sturctures that are associated
 * with our Ir Port.
 *
 * PARAMETERS:   None
 *
 * RETURNED:     None
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
void IrClosePort()
{
	if (!serIsOpen)
		return;
		
	// We will first flush both receive and send buffers.
	// This is also present as a safeguard. In some version of the
	// Old Serial Manager, an extra character was sent when opening
	// and closing a port.
	
	if (serMgrNew)
	{
		SrmSendFlush(irPortId);
		SrmReceiveFlush(irPortId, 0);
		SrmClose(irPortId);
	}
	else
	{
		SerSendFlush(irPortId);
		SerReceiveFlush(irPortId, 0);
		SerClose(irPortId);
	}
}