/***********************************************************************
 * Author: 	Kevin Chan
 *
 * Description:
 * 		The code present provides an example of the serial interface
 * 		that is used. These are light wrappers in the sense that error
 *      codes are returned directly back to you. You can control the
 *      serial port to if you want as the reference number will be
 *      returned to you.
 *
 *      The only safe condition would possibly be to configure the port;
 *
 * NOTE: The API will multiplex between the old and new Serial Managers
 * if the new Serial Manager is present, that will be the prefered 
 * Manager.
 *
 * It is also dependent on AppStdIO for debugging. Because pre m500
 * devices only have one UART, we cannot live debug the system. So
 * we create a console equivalent (i.e. AppStdIO).
 ***********************************************************************/
 
#ifndef __SERIAL_IR_H__
#define __SERIAL_IR_H__

#include <SerialMgrOld.h>

// This is the default queue size
// in all of the serial managers.
#define MAX_RCV_QUEUE_SIZE 512

// Comment out the following API if you do not want to compile in the
// debugging code.
#define __DEBUG_IR_MSGS

/***********************************************************************
 *
 * FUNCTION:     DebugDisplaySerErrMsg
 *
 * DESCRIPTION:  Converts a Serial Error Message and displays the textual
 * equivalent.
 *
 * PARAMETERS:   --> serErr  // The Serial Error Number
 *
 * RETURNED:     Err         // Returns the serErr it received.
 ***********************************************************************/
Err DebugDisplaySerErrMsg(Err serErr);
 
/***********************************************************************
 *
 * FUNCTION:     DebugDisplayLineErrMsg
 *
 * DESCRIPTION:  Given a line err number, it will convert it to the
 * text equivalent and output a message.
 *
 * NOTE: This function does not clear the line errors. You must
 * clear them before continuing with a call to get data from the UART.
 *
 * PARAMETERS:   UInt16    // the Line Err number
 *
 * RETURNED:     UInt16    // The lineErr number.
 ***********************************************************************/
UInt16 DebugDisplayLineErrMsg(UInt16 lineErrs);

/***********************************************************************
 *
 * FUNCTION:     IrOpenPort
 *
 * DESCRIPTION:  It will attempt to bind itself to a serial line through
 * the UART. More specifically, it will bind to the IR Port. It will
 * also determine whether the New Serial Manager is present and if it
 * is then it will use that library instead.
 *
 * PARAMETERS:   --> baud    // The baud rate you want on the port
 *
 * RETURNED:    errNone				 // No error
 *				serErrAlreadyOpen	 // Port is already opened and port
 *									 // port sharing is not allowed.
 *				serErrBadParam		 // baud rate was invalid.
 *				memErrNotEnoughSpace // Insufficient memory
 *				
 *
 ***********************************************************************/
Err IrOpenPort(UInt32 baud);


/***********************************************************************
 *
 * FUNCTION:     IrGetSerialLibRefNum
 *
 * DESCRIPTION:  Returns the IR Port ID. The Port ID is serial manager
 * specific. If the new serial manager is present, then the Port ID is
 * should be used as the "portID" parameter for the New Serial Manager
 * APIs.
 *
 * Else, the Port ID should be used as the refNum parameter for the Old
 * Serial Manager APIs (only if the New Serial Manager is not present.)

 * This is useful if you want/need to change the IR port configurations
 *
 * RETURNED:     UInt16  // The library reference number.
 ***********************************************************************/
UInt16 GetIrPortID();


/***********************************************************************
 *
 * FUNCTION:     IsUsingNewSerialManager
 *
 * DESCRIPTION:  Helps determine which serial manager is being used.
 *
 * PARAMETERS:   True (new serial manager v1 is being used)
 *               False (old serial manger is being used.)
 *
 * RETURNED:     UInt16  // The library reference number.
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
Boolean IsUsingNewSerialManager();


/***********************************************************************
 *
 * FUNCTION:     IrSendData
 *
 * DESCRIPTION:  A wrapper that wraps the SerSend functionality.
 * Specifically it will send the data you specify and then wait for all
 * of the data to be sent before returnning.
 *
 * NOTE: delivery is not guaranteed. In another words if the receiver
 * experiences a line error (oh well). If you want reliable delivery
 * use the other serial port functionality provided by the PALM or right
 * your own stack on top of these APIs.
 *
 * NOTE 2: On older palms, there is a feedback problem. Thus a call is
 * made to flush the internal receive buffer.
 *
 * PARAMETERS:  -->buffP    // ptr to the buffer containing the data.
 *              -->count    // size of buffP in bytes.
 *              <--bytesTxd // The number of bytes that were sent.
 *
 * RETURNED:    errNone     // No Error
 *
 *				serErrNotOpen // You forgot to call IrOpenPort
 *
 *				serErrTimeOut // Unable to send data within the specified
 *							  // CTS timeout period. (You can change the
 *							  // the Timeout Value
 *
 *				serConfigurationFailed 	// You changed the port configuration
 *
 *				serErrNotSuported 		// You changed the port to be a background port
 *
 *
 ***********************************************************************/
Err IrSendData(void* buffP, UInt32 count, UInt32* bytesTxd );


/***********************************************************************
 *
 * FUNCTION:     IrReceiveFlush
 *
 * DESCRIPTION:  A wrapper around the SerReceiveFlush function
 *
 * PARAMETERS:   -->timeout  // After the timeout amount, SerReceiveFlush
 *                           // will execute.
 ***********************************************************************/
void IrReceiveFlush(UInt32 timeout);



/***********************************************************************
 *
 * FUNCTION:     IrReceiveData
 *
 * DESCRIPTION:  This will attempt to read whatever is in the receive
 * buffer of the UART. It wraps the SerReceive Functionality.
 * Specifically, it will block until count bytes or a timeout occurred.
 *
 * if a timeout occurs first, we will return whatever bytes is sitting
 * in the buffer.
 *
 * PARAMETERS:   <->buffP     // the buffer to contain the received data.
 *               -->count     // Size of buffP in bytes
 *               <--bytesRcvd // The # of bytes that was received.
 *               -->timeout   // Timeout amount in system ticks. Actual
 *               <--lineErrCode // The Line Error code if a line error occurred.
 *
 * RETURNED:     errNone		// No Error
 *				 serErrNotOpen 	// You forgot to call IrOpenPort
 *				 serErrLineErr	// An RS-232 Line error occurred. See
 *								// lineErrCode parameter for the code.
 *				 serErrNotSupported // You changed the port to a background port
 *
 *				 serErrBadParam	// The bytes parameter exceeds the size of
 *								// Receive buffer queue.
 *
 * NOTE: for the old serial manager, input buffer is only 512 bytes. so
 * if you want more than 512 bytes, you are out of luck. If you try to 
 * pass more, the code will scale this back down to 512.
 *
 * Same thing goes for the new serial manager unless you change the
 * input buffer which you can do only in the new serial manager. You must
 * make this call yourself using SrmSetReceiveBuffer.
 ***********************************************************************/
Err IrReceiveData(void* buffP, UInt32 count, UInt32* bytesRcvd, UInt32 timeout, UInt16 *lineErrCode);

/***********************************************************************
 *
 * FUNCTION:     IrReceiveCheck
 *
 * DESCRIPTION:  Light wrapper around the (Srm/Ser)ReceiveCheck function
 *
 * PARAMETERS:   <--bytes   // The number of bytes in the receive buffer.
 *
 * RETURNED:     errNone		 // No Error
 *				 serErrNotOpen   // You forgot to call IrOpenPort
 *				 serErrLineError // A Line Error Occurred, check the
 *								 // lineErrCode paramater
 ***********************************************************************/
Err IrReceiveCheck(UInt32* bytes, UInt16 *lineErrCode);

/***********************************************************************
 * FUNCTION:     IrClosePort
 *
 * DESCRIPTION:  Releases all of the data sturctures that are associated
 * with our Ir Port.
 ***********************************************************************/
void IrClosePort();

#endif //__SERIAL_IR_H__